import { navigateTo } from "#app"
import { useUserinfoStore, Userinfo } from "@/stores/UserinfoStore"
import { $fetch } from 'ohmyfetch'

export type AllowRole = '' | 'player' | 'gamemaster' | 'admin'
export type AllowRoles = Array<AllowRole>

export interface useAuth {
  login(authCode: string): Promise<void>
  logout(): Promise<void>
  authenticateAndLoadUserInfo(allowRoles: AllowRoles): Promise<void>
}

export default (): useAuth => {
  const user = useUserinfoStore()

  return {
    authenticateAndLoadUserInfo: async (allowRoles: AllowRoles): Promise<void> => {
      try {
        const userInfo = await $fetch('/api/userinfo') as Userinfo
        user.setUserInfo(userInfo)
        if (allowRoles.indexOf(userInfo.role) >= 0 ) {
          return
        }
        if (user.isAdmin) {
          navigateTo('/admin', { replace: true })
        } else {
          navigateTo('/play', { replace: true })
        }
      } catch (e) {
        if (allowRoles.indexOf('') == -1 ) {
          navigateTo('/', { replace: true })
        }
      }
    },

    login: async (authCode: string): Promise<void> => {
      if (authCode.length != 6) {
        throw Error('login failed')
      }
      const resp = await $fetch(`/api/login?code=${authCode}`)
      if (resp !== 'ok') {
        throw Error('login failed')
      }
    },

    logout: async (): Promise<void> => {
      await $fetch('/api/logout')
    },
  }
}
